package com.secutix.plugin.urlshortener.google;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Joiner;
import com.google.common.base.Strings;
import com.google.common.collect.ImmutableMap;
import com.secutix.plugin.util.LogLevel;
import com.secutix.plugin.util.StxPluginHttpResponse;
import com.secutix.plugin.v1_0.dto.PluginCallResult;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class GoogleUrlShortenerPlugin extends UrlShortenerPlugin {


    public static final String ID = "id";
    private ObjectMapper jsonMapper = new ObjectMapper();

    @Override
    public String getInterfaceType() {
        return "US_GOOGLE";
    }

    @Override
    public Map<String, String> getInterfaceNamesByLanguageCode() {
        return ImmutableMap.of("en", "Google url shortener plugin", "fr", "Plugin url shortener Google");
    }



    @Override
    public ShortUrlCallResult shortenUrl(String longUrl) {
        //Validating the parameters.
        String url = getInterfaceParametersProvider().getUrl();
        String apiKey = getInterfaceParametersProvider().getPassword();
        if (Strings.isNullOrEmpty(url)) {
            logForOperator(LogLevel.ERROR, "[en] url not defined. Please define one",
                    "[fr] url non définie. Veuillez en définir une.");
            return PluginCallResult.addFailure(new ShortUrlCallResult(), "invalid.parameter", "No URL defined");
        }
        if (Strings.isNullOrEmpty(apiKey)) {
            logForOperator(LogLevel.ERROR, "[en] ApiKey not defined. Please store your google ApiKey in password field.",
                    "[fr] ApiKey non définie. Veuillez enregistrer votre clef d'Api Google dans le champ mot de passe.");
            return PluginCallResult.addFailure(new ShortUrlCallResult(), "invalid.parameter", "No API Key defined");
        }
        getLogger().debug("All parameters properly defined");

        if (Strings.isNullOrEmpty(longUrl)) {
            logForOperator(LogLevel.ERROR, "[en] cannot shorten empty url.",
                    "[fr] Ne peux raccourcir une url vide.");
            return PluginCallResult.addFailure(new ShortUrlCallResult(), "invalid.parameter", "Invalid url :" + longUrl);
        }


        String urlWithApiKey = url + "?key=" + apiKey;
        //urlWithApiKey = url ;
        Map<String, String> values = ImmutableMap.of("longUrl", longUrl);
        try {
            StxPluginHttpResponse res = getHttpClientHelper().doPost(urlWithApiKey, values, ImmutableMap.of("Content-Type","application/json"));
            if (res.getStatusCode() != StxPluginHttpResponse.HTTP_200) {
                logForOperator(LogLevel.ERROR, "[en] Failed shortening url :" + longUrl + " " + res, "[fr] Erreur en raccourcissant l'url :" + longUrl + " " + res);
                return PluginCallResult.addFailure(new ShortUrlCallResult(), "http.error", "" + res);
            }
            Map<String, String> responseValues = jsonMapper.readValue(res.getResponseData(), Map.class);
            if (!responseValues.containsKey(ID)) {
                logForOperator(LogLevel.ERROR, "[en] Failed shortening url :" + longUrl + ". Response does not contain any id. " + res.getResponseData(), "[fr] Erreur en raccourcissant l'url :" + longUrl + ". La réponse ne contient pas la clef id. " + res.getResponseData());
                return PluginCallResult.addFailure(new ShortUrlCallResult(), "invalid.response", "" + res.getResponseData());
            }
            String shortenedUrl = responseValues.get(ID);
            getLogger().info("" + longUrl + " -> " + shortenedUrl);
            ShortUrlCallResult finalResult = new ShortUrlCallResult();
            finalResult.setShortUrl(shortenedUrl);
            return PluginCallResult.addSucess(finalResult,"");
        } catch (Exception e) {
            logForOperator(LogLevel.ERROR, "[en] Technical error when calling google APIs:" + e.getMessage(),
                    "[fr] Erreur technicque en appelant l'API google:" + e.getMessage());
            getLogger().error("Error when calling google APIs.", e);
            return PluginCallResult.addFailure(new ShortUrlCallResult(), "technical error", "" + e.getMessage());
        }
    }
}
